@@ -87,7 +87,7 @@ add_clicked_cb (GtkButton *button,
 		g_free (message);
 	} else {		
 		gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
-		g_signal_emit_by_name (nautilus_signaller_get_current (), "mime_data_changed");
+		g_signal_emit_by_name (nautilus_signaller_get_current (), "mime-data-changed");
 	}
 
 	g_object_unref (info);
@@ -119,7 +119,7 @@ remove_clicked_cb (GtkMenuItem *item,
 		g_object_unref (info);
 	}
 
-	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime_data_changed");
+	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime-data-changed");
 }
 
 static void
@@ -152,7 +152,7 @@ reset_clicked_cb (GtkButton *button,
 	g_app_info_reset_type_associations (chooser->details->content_type);
 	gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
 
-	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime_data_changed");
+	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime-data-changed");
 }
 
 static void
@@ -180,7 +180,7 @@ set_as_default_clicked_cb (GtkButton *button,
 	g_object_unref (info);
 
 	gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
-	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime_data_changed");
+	g_signal_emit_by_name (nautilus_signaller_get_current (), "mime-data-changed");
 }
 
 static gint
