@@ -750,7 +750,7 @@ on_connect_server_response (GtkDialog      *dialog,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-static void
+GtkWidget *
 nautilus_application_connect_server (NautilusApplication *application,
 				     NautilusWindow      *window)
 {
@@ -770,6 +770,8 @@ nautilus_application_connect_server (NautilusApplication *application,
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
 	gtk_window_set_screen (GTK_WINDOW (dialog), gtk_window_get_screen (GTK_WINDOW (window)));
 	gtk_window_present (GTK_WINDOW (dialog));
+
+	return dialog;
 }
 
 static void
