@@ -359,7 +359,7 @@ nautilus_desktop_canvas_view_handle_middle_click (NautilusCanvasContainer *canva
 
 	/* Stop the event because we don't want anyone else dealing with it. */	
 	gdk_flush ();
-	g_signal_stop_emission_by_name (canvas_container, "middle_click");
+	g_signal_stop_emission_by_name (canvas_container, "middle-click");
 
 	/* build an X event to represent the middle click. */
 	x_event.type = ButtonPress;
@@ -491,7 +491,7 @@ delayed_init (NautilusDesktopCanvasView *desktop_canvas_view)
 {
 	/* Keep track of the load time. */
 	g_signal_connect_object (nautilus_view_get_model (NAUTILUS_VIEW (desktop_canvas_view)),
-				 "done_loading",
+				 "done-loading",
 				 G_CALLBACK (done_loading), desktop_canvas_view, 0);
 
 	/* Monitor desktop directory. */
@@ -554,7 +554,7 @@ nautilus_desktop_canvas_view_init (NautilusDesktopCanvasView *desktop_canvas_vie
 	 */
 	if (!nautilus_monitor_active ()) {
 		desktop_canvas_view->details->delayed_init_signal = g_signal_connect_object
-			(desktop_canvas_view, "begin_loading",
+			(desktop_canvas_view, "begin-loading",
 			 G_CALLBACK (delayed_init), desktop_canvas_view, 0);
 	}
 	
@@ -582,7 +582,7 @@ nautilus_desktop_canvas_view_init (NautilusDesktopCanvasView *desktop_canvas_vie
 	nautilus_view_set_show_foreign (NAUTILUS_VIEW (desktop_canvas_view),
 					FALSE);
 	
-	g_signal_connect_object (canvas_container, "middle_click",
+	g_signal_connect_object (canvas_container, "middle-click",
 				 G_CALLBACK (nautilus_desktop_canvas_view_handle_middle_click), desktop_canvas_view, 0);
 	g_signal_connect_object (desktop_canvas_view, "realize",
 				 G_CALLBACK (realized_callback), desktop_canvas_view, 0);
